﻿------------------------------
--      Are you local?      --
------------------------------

local seaura = AceLibrary("SpecialEvents-Aura-2.0")
local semount = AceLibrary("SpecialEvents-Mount-2.0")
local tablet = AceLibrary("Tablet-2.0")
local crayon = AceLibrary("Crayon-2.0")

local _, myclass = UnitClass("player")
local spellmounts = {
	[(GetSpellInfo(5784))] = 60, -- Felsteed (Warlock)
	[(GetSpellInfo(13819))] = 60, -- Warhorse (Paladin)
	[(GetSpellInfo(23161))] = 100, -- Dreadsteed (Warlock)
	[(GetSpellInfo(23214))] = 100 -- Charger (Paladin)
}
local defaulticon = "Interface\\Icons\\Ability_Kick"
local buffs, boosts, mountused, mmitemswap = {}, {}
local itemslots = {carrot = 13, crop = 13, whip = 13, spurs = 8, gloves = 10}
local itemboosts = {carrot = 3, crop = 10, whip = 10, spurs = 4, gloves = 2}
local invslots = {[8] = true, [10] = true, [13] = true, [14] = true, [true] = true}
local buffspeeds = {
	[(GetSpellInfo(13159))] = 30, -- Aspect of the Pack
	[(GetSpellInfo(5118))] = 30, -- Aspect of the Cheetah
	[(GetSpellInfo(2645))] = 40, -- Ghost Wolf
	[(GetSpellInfo(783))] = 40, -- Travel Form
	[(GetSpellInfo(1066))] = 50, -- Aquatic Form
	[(GetSpellInfo(32223))] = 20, -- Crusader Aura
	[(GetSpellInfo(33943))] = 60, -- Flight Form
	[(GetSpellInfo(40120))] = 280, -- Swift Flight Form
}


-------------------------------------
--      Namespace Declaration      --
-------------------------------------

MountUp = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "FuBarPlugin-2.0")
MountUp:RegisterDB("MountUpDB", "MountUpDBPerChar")
MountUp.cmdtable = {type = "group", handler = MountUp, args = {}}
MountUp:RegisterChatCommand({"/mu", "/mountup"}, MountUp.cmdtable)
MountUp.hasIcon = defaulticon
MountUp.independentProfile = true
MountUp.hideWithoutStandby = true
MountUp.OnMenuRequest = MountUp.cmdtable


---------------------------
--      Ace Methods      --
---------------------------

function MountUp:OnInitialize()
	
	self.revision = tonumber(("$Revision: 69346 $"):match("%d+"))
	self.version = self.version .. "." .. self.revision

	for name,module in self:IterateModules() do
		if module.consoleOptions then
			self.cmdtable.args[module.consoleCmd] = module.consoleOptions
		end
	end
end


function MountUp:OnEnable()
	mmitemswap = self:HasModule("ItemSwap") and self:GetModule("ItemSwap")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", SetMapToCurrentZone)
	self:RegisterEvent("SpecialEvents_Mounted")
	self:RegisterEvent("SpecialEvents_Dismounted")
	self:RegisterEvent("SpecialEvents_PlayerBuffGained")
	self:RegisterEvent("SpecialEvents_PlayerBuffLost")
	self:RegisterEvent("MountUp_Deshift")
	self:RegisterEvent("SpecialEvents_EquipmentChanged")
end


-----------------------------
--      FuBar Methods      --
-----------------------------

function MountUp:OnTextUpdate()
	local s = 100
	local i = 0
	for _,val in pairs(buffs) do s = s + val end
	for _,val in pairs(boosts) do i = i + val end
	s = s * (100 + i) / 100
	self:SetText(string.format("|cff%s%s%%|r", crayon:GetThresholdHexColor(s, 0, 60, 75, 100, 125), s))
end


function MountUp:OnTooltipUpdate()
	local cat = tablet:AddCategory("text", "Buffs", "columns", 2)
	for i,val in pairs(buffs) do cat:AddLine("text", i, "text2", string.format("+%d%%", val)) end

	local cat2 = tablet:AddCategory("text", "Boosts", "columns", 2)
	for i,val in pairs(boosts) do
		local link = GetInventoryItemLink("player", itemslots[i])
		if link then _, _, link = string.find(link, "(item:%d+:%d+:%d+:%d+)") end
		local name = link and GetItemInfo(link)
		cat2:AddLine("text", name, "text2", string.format("+%d%%", val))
	end
end


------------------------------
--      Event Handlers      --
------------------------------


function MountUp:SpecialEvents_Mounted(buff, speed, idx)
	buffs[buff] = speed
	self:SpecialEvents_EquipmentChanged(true)
	self:SetIcon(GetPlayerBuffTexture(GetPlayerBuff(idx, "HELPFUL")))
	self:Update()
end


function MountUp:SpecialEvents_Dismounted(buff)
	mountused = nil
	buffs[buff] = nil
	self:SpecialEvents_EquipmentChanged(true)
	self:SetIcon(defaulticon)
	self:Update()
end


function MountUp:SpecialEvents_PlayerBuffGained(buff, idx)
	if buffspeeds[buff] then
		buffs[buff] = buffspeeds[buff]
		self:SetIcon(GetPlayerBuffTexture(GetPlayerBuff(idx, "HELPFUL")))
		self:Update()
	end
end


function MountUp:SpecialEvents_PlayerBuffLost(buff)
	if buffs[buff] then self:SetIcon(defaulticon) end
	buffs[buff] = nil
	self:Update()
end


function MountUp:SpecialEvents_EquipmentChanged(i)
	if not mmitemswap then return end

	if invslots[i] then
		local mounted = semount:PlayerMounted()
		for item,val in pairs(itemslots) do
			boosts[item] = mounted and mmitemswap:CheckItem(item, val) and itemboosts[item]
		end
		if i ~= true then self:Update() end
	end
end


function MountUp:MountUp_Deshift()
	if myclass == "DRUID" then
		for i=1,GetNumShapeshiftForms() do
			local _, name, active = GetShapeshiftFormInfo(i)
			-- if active and not flight form
			if active and not (name == (GetSpellInfo(33943)) or name == (GetSpellInfo(40120))) then
				CastShapeshiftForm(i)
				return true
			end
		end
	elseif myclass == "PRIEST" then
		-- Shadowform
		local i = GetPlayerBuffName((GetSpellInfo(15473)))
		if i then
			CancelPlayerBuff(i)
			return true
		end
	elseif myclass == "SHAMAN" then
		-- Ghost Wolf
		local i = GetPlayerBuffName((GetSpellInfo(2645)))
		if i then
			CancelPlayerBuff(i)
			return true
		end
	end
end
