----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("MountUp ItemSwap")

L:RegisterTranslations("enUS", function() return {
	rein = "Reindeer",

	swap = true,
	Swap = true,
	["Options for automatic speed item swapping."] = true,

	temp = true,
	["Suspend temporarily"] = true,
	["Suspend item swaps temporarily, this setting does not persist across sessions."] = true,

	pvp = true,
	["Suspend when PvP"] = true,
	["Suspend item swaps when the player is PvP flagged."] = true,

	bg = true,
	["Suspend in BG"] = true,
	["Suspend item swaps when the player is in a battleground."] = true,

	raid = true,
	["Suspend when raiding"] = true,
	["Suspend item swaps when the player is in a Raidzone."] = true,
} end)

L:RegisterTranslations("esES", function() return {
	rein = "Renos",

	swap = "intercambiar",
	Swap = "Intercambiar",
	["Options for automatic speed item swapping."] = "Opciones para velocidad de cambio de elemento autom\195\161tico.",

	temp = "temp",
	["Suspend temporarily"] = "Suspender temporalmente",
	["Suspend item swaps temporarily, this setting does not persist across sessions."] = "Suspende intercambio de elemento temporalmente, esta opci\195\179n no persiste entre sesiones.",

	pvp = "JcJ",
	["Suspend when PvP"] = "Suspender en JcJ",
	["Suspend item swaps when the player is PvP flagged."] = "Suspende intercambio de elementos cuando el jugador est\195\161 en modo JcJ.",

	bg = "CB",
	["Suspend in BG"] = "Suspender en CB",
	["Suspend item swaps when the player is in a battleground."] = "Suspende intercambio de elementos cuando el jugador est\195\161 en un Campo de Batalla.",

	raid = "Raid",
	["Suspend when raiding"] = "Suspender al raidear",
	["Suspend item swaps when the player is in a Raidzone."] = "Suspende intercambio de elementos cuando el jugador est\195\161 en una Zona de Raid.",
} end)

L:RegisterTranslations("koKR", function() return {
	rein = "순록",

	swap = "교체",
	Swap = "아이템 교체",
	["Options for automatic speed item swapping."] = "자동으로 탈것의 속도를 증가시키는 아이템으로 교체해주는 옵션입니다.",

	temp = "임시",
	["Suspend temporarily"] = "일시 중지",
	["Suspend item swaps temporarily, this setting does not persist across sessions."] = "일시적으로 아이템 교체를 중지합니다. 이 설정은 저장되지 않고 접속 중에만 유지됩니다.",

	pvp = "pvp",
	["Suspend when PvP"] = "PvP 전투 플래그 활성화 시 중지",
	["Suspend item swaps when the player is PvP flagged."] = "플레이어가 PvP 전투 플래그를 활성화했을 경우 아이템 교체를 중지합니다.",

	bg = "전장",
	["Suspend in BG"] = "전장에서 중지",
	["Suspend item swaps when the player is in a battleground."] = "플레이어가 전장에 있으면 아이템 교체를 중지합니다.",

	--raid = true,
	--["Suspend when raiding"] = true,
	--["Suspend item swaps when the player is in a Raidzone."] = true,
} end)


------------------------------
--      Are you local?      --
------------------------------

local seequip = AceLibrary("SpecialEvents-Equipped-2.0")
local sebags = AceLibrary("SpecialEvents-Bags-2.0")
local semount = AceLibrary("SpecialEvents-Mount-2.0")
local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable()

local items, delayed, incombat, raindelay, tempdisable = {}
local itemslots = {carrot = 13, crop = 13, whip = 13, spurs = 8, gloves = 10}
local itemstrs = {carrot = "item:11122:%d+:%d+:%d+", crop = "item:25653:%d+:%d+:%d+", whip = "item:32863:%d+:%d+:%d+", spurs = "item:%d+:464:%d+:%d+", gloves = "item:%d+:930:%d+:%d+"}
local battlegrounds = {
	[BZ["Warsong Gulch"]] = true,
	[BZ["Arathi Basin"]] = true,
	[BZ["Alterac Valley"]] = true,
	[BZ["Eye of the Storm"]] = true,
	[BZ["Blade's Edge Arena"]] = true,
	[BZ["Nagrand Arena"]] = true,
	[BZ["Ruins of Lordaeron"]] = true,
}
local raidzones = {
	[BZ["Hyjal Summit"]] = true,
	[BZ["Black Temple"]] = true,
	[BZ["Zul'Aman"]] = true,
}
local excludebuffs = {
	(GetSpellInfo(13159)), -- Aspect of the Pack
	(GetSpellInfo(5118)), -- Aspect of the Cheetah
	(GetSpellInfo(2645)), -- Ghost Wolf
	(GetSpellInfo(783)), -- Travel Form
	(GetSpellInfo(1066)), -- Aquatic Form
	(GetSpellInfo(32223)), -- Crusader Aura
	(GetSpellInfo(33943)), -- Flight Form
	(GetSpellInfo(40120)), -- Swift Flight Form
	(GetSpellInfo(1850)), -- Dash
	(GetSpellInfo(2983)), -- Sprint
	(GetSpellInfo(31641)), -- Blazing Speed
}

-------------------------------------
--      Namespace Declaration      --
-------------------------------------

MountUpItemSwap = MountUp:NewModule("ItemSwap", "AceEvent-2.0", "AceDebug-2.0")
MountUp:RegisterDefaults("ItemSwap", "char", {
	BGsuspend = true,
	RAIDsuspend = true,
	PvPsuspend = false,
})
MountUpItemSwap.db = MountUp:AcquireDBNamespace("ItemSwap")
MountUpItemSwap.consoleCmd = L["swap"]
MountUpItemSwap.consoleOptions = {
	type = "group",
	name = L["Swap"],
	desc = L["Options for automatic speed item swapping."],
	args = {
		[L["temp"]] = {
			type = "toggle",
			name = L["Suspend temporarily"],
			desc = L["Suspend item swaps temporarily, this setting does not persist across sessions."],
			get = function() return tempdisable end,
			set = function(v) tempdisable = v end,
		},
		[L["pvp"]] = {
			type = "toggle",
			name = L["Suspend when PvP"],
			desc = L["Suspend item swaps when the player is PvP flagged."],
			get = function() return MountUpItemSwap.db.char.PvPsuspend end,
			set = function(v) MountUpItemSwap.db.char.PvPsuspend = v end,
		},
		[L["bg"]] = {
			type = "toggle",
			name = L["Suspend in BG"],
			desc = L["Suspend item swaps when the player is in a battleground."],
			get = function() return MountUpItemSwap.db.char.BGsuspend end,
			set = function(v) MountUpItemSwap.db.char.BGsuspend = v end,
		},
		[L["raid"]] = {
			type = "toggle",
			name = L["Suspend when raiding"],
			desc = L["Suspend item swaps when the player is in a Raidzone."],
			get = function() return MountUpItemSwap.db.char.RAIDsuspend end,
			set = function(v) MountUpItemSwap.db.char.RAIDsuspend = v end,
		},
	}
}


---------------------------
--      Ace Methods      --
---------------------------

function MountUpItemSwap:OnEnable()
	self:ScanInventory()
	self:RegisterEvent("SpecialEvents_Mounted")
	self:RegisterEvent("SpecialEvents_Dismounted")
	self:RegisterEvent("SpecialEvents_BagSlotUpdate")
	self:RegisterEvent("SpecialEvents_EquipmentChanged")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end


------------------------------
--      Event Handlers      --
------------------------------

function MountUpItemSwap:SpecialEvents_Mounted(mount)
	for k=1, table.getn(excludebuffs) do
		if string.find(mount, excludebuffs[k]) then
			return
		end
	end
	if mount == L.rein then raindelay = true
	else self:Swap() end
end


function MountUpItemSwap:SpecialEvents_Dismounted()
	if incombat then delayed = true
	else self:Swap(true) end
end


function MountUpItemSwap:PLAYER_REGEN_DISABLED()
	if not raindelay then incombat = true end
end


function MountUpItemSwap:PLAYER_REGEN_ENABLED()
	if raindelay then raindelay = nil
	elseif delayed then self:Swap(true) end
	incombat = nil
	delayed = nil
end


function MountUpItemSwap:SpecialEvents_EquipmentChanged()
	if raindelay and not incombat then
		raindelay = nil
		self:Swap()
	end
end


function MountUpItemSwap:SpecialEvents_BagSlotUpdate(bag, slot)
	local itype = self:CheckItem(bag, slot)

	if itype then
		items[itype] = {bag, slot}
	else
		for i in pairs(itemstrs) do
			local s = items[i]
			if s and s[1] == bag and s[2] == slot and not self:CheckItem(i, itemslots[i]) then
				items[i] = nil
				return
			end
		end
	end
end


-----------------------------------
--      Speed item swapping      --
-----------------------------------

function MountUpItemSwap:IsSuspended()
	return tempdisable
		or self.db.char.PvPsuspend and UnitIsPVP("player")
		or self.db.char.BGsuspend and battlegrounds[GetZoneText()]
		or self.db.char.RAIDsuspend and raidzones[GetZoneText()]
end


function MountUpItemSwap:Swap(reset)
	if self:IsSuspended() then return
	elseif reset then
		for i in pairs(itemstrs) do
			if items[i] and self:CheckItem(i, itemslots[i]) then self:EquipItem(items[i][1], items[i][2]) end
		end
	else
		for i in pairs(itemstrs) do
			if items[i] and not self:CheckItem(i, itemslots[i]) then
				self:EquipItem(items[i][1], items[i][2])
				self.db.char[i] = GetInventoryItemLink("player", itemslots[i])
				self:EquipItem(items[i][1], items[i][2])
			end
		end
	end
end


function MountUpItemSwap:EquipItem(bag, slot)
	if not GetContainerItemLink(bag, slot) then return end
	self:Debug("Equipping", GetContainerItemLink(bag, slot))
	PickupContainerItem(bag, slot)
	AutoEquipCursorItem()
end


----------------------------------
--			Speed item tracking			--
----------------------------------

function MountUpItemSwap:ScanInventory()
	for bag=0,4 do
		for slot=1,GetContainerNumSlots(bag) do
			local link = GetContainerItemLink(bag, slot)
			if link then
				for i in pairs(itemstrs) do
					if self.db.char[i] and link == self.db.char[i] and self:CheckItem(i, itemslots[i]) then
						items[i] = {bag, slot}
						if not semount:PlayerMounted() then self:EquipItem(bag, slot) end
					end
				end

				local itemtype = self:CheckItem(bag, slot)
				if itemtype then items[itemtype] = {bag, slot} end
			end
		end
	end
end


function MountUpItemSwap:CheckItem(bag, slot)
	assert(bag and slot, "You must pass two args!")

	if type(bag) == "string" then
		assert(itemstrs[bag], "Invalid item type: "..bag)
		local link = GetInventoryItemLink("player", slot)
		if link and string.find(link, itemstrs[bag]) then return true end
	else
		for item,str in pairs(itemstrs) do
			local link = GetContainerItemLink(bag, slot)
			if link and string.find(link, str) then return item end
		end
	end
end